@@ -14,6 +14,10 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <string.h>
+#ifdef TARGET_OS_HAIKU
+#include <posix/string.h>
+#include <bsd/string.h>
+#endif
 #include <unistd.h>
 #include <sys/stat.h>
 #include <sys/time.h>
@@ -26,6 +30,7 @@
 #include <arpa/inet.h>
 #include <ctype.h>
 
+#ifndef USE_POLARSSL
 #include <openssl/bio.h>
 #include <openssl/ssl.h>
 #include <openssl/err.h>
@@ -33,8 +38,10 @@
 #include <openssl/x509.h>
 #include <openssl/conf.h>
 #include <openssl/x509v3.h>
+#endif
 
 int verbose;
+int verbose_debug;
 
 #include "src/util.h"
 
@@ -83,6 +90,19 @@ int verbose;
 // To support our RFC 2595 wildcard verification
 #define RFC2595_MIN_LABEL_COUNT 3
 
+// Define a max length for the HTTP Date: header
+#define MAX_DATE_LINE_LEN 32
+
+// Define a max length for HTTP headers
+#define MAX_HTTP_HEADERS_SIZE 8192
+
+// Define our basic HTTP request
+#define HTTP_REQUEST    \
+  "HEAD / HTTP/1.1\r\n" \
+  "User-Agent: %s\r\n"  \
+  "Host: %s\r\n"        \
+  "\r\n"
+
 static int ca_racket;
 
 static const char *host;
@@ -96,6 +116,7 @@ static const char *protocol;
 static char *proxy;
 
 static const char *ca_cert_container;
+#ifndef USE_POLARSSL
 void openssl_time_callback (const SSL* ssl, int where, int ret);
 uint32_t get_certificate_keybits (EVP_PKEY *public_key);
 uint32_t check_cn (SSL *ssl, const char *hostname);
@@ -105,12 +126,12 @@ uint32_t check_name (SSL *ssl, const char *hostname);
 uint32_t verify_signature (SSL *ssl, const char *hostname);
 void check_key_length (SSL *ssl);
 void inspect_key (SSL *ssl, const char *hostname);
-static void run_ssl (uint32_t *time_map, int time_is_an_illusion);
 void check_key_length (SSL *ssl);
+void inspect_key (SSL *ssl, const char *hostname);
+#endif
 uint32_t dns_label_count (char *label, char *delim);
 uint32_t check_wildcard_match_rfc2595 (const char *orig_hostname,
                                        const char *orig_cert_wild_card);
-void inspect_key (SSL *ssl, const char *hostname);
-static void run_ssl (uint32_t *time_map, int time_is_an_illusion);
+static void run_ssl (uint32_t *time_map, int time_is_an_illusion, int http);
 
 #endif
