@@ -91,10 +91,11 @@ usage(void)
           " [-P|--protocol] [sslv23|sslv3|tlsv1]\n"
           " [-C|--certcontainer] [dirname|filename]\n"
           " [-v|--verbose]\n"
-          " [-V|--showtime]\n"
+          " [-V|--showtime] [human|raw]\n"
           " [-t|--timewarp]\n"
           " [-l|--leap]\n"
-    " [-x|--proxy] [url]\n");
+          " [-x|--proxy] [url]\n"
+          " [-w|--http]\n");
 }
 
 
@@ -112,6 +113,7 @@ main(int argc, char **argv)
   int timewarp;
   int leap;
   const char *proxy;
+  int http;
 
   host = DEFAULT_HOST;
   port = DEFAULT_PORT;
@@ -124,6 +126,7 @@ main(int argc, char **argv)
   timewarp = 0;
   leap = 0;
   proxy = NULL;
+  http = 0;
 
   while (1) {
     int option_index = 0;
@@ -132,7 +135,7 @@ main(int argc, char **argv)
     static struct option long_options[] =
       {
         {"verbose", 0, 0, 'v'},
-        {"showtime", 0, 0, 'V'},
+        {"showtime", 2, 0, 'V'},
         {"skip-verification", 0, 0, 's'},
         {"help", 0, 0, 'h'},
         {"host", 0, 0, 'H'},
@@ -143,17 +146,18 @@ main(int argc, char **argv)
         {"timewarp", 0, 0, 't'},
         {"leap", 0, 0, 'l'},
         {"proxy", 0, 0, 'x'},
+        {"http", 0, 0, 'w'},
         {0, 0, 0, 0}
       };
 
-    c = getopt_long(argc, argv, "vVshH:p:P:nC:tlx:",
+    c = getopt_long(argc, argv, "vV::shH:p:P:nC:tlx:w",
                     long_options, &option_index);
     if (c == -1)
       break;
 
     switch (c) {
-      case 'v': verbose = 1; break;
-      case 'V': showtime = 1; break;
+      case 'v': verbose += 1; break;
+      case 'V': showtime = (optarg && 0 == strcmp("raw", optarg) ? 2:1); break;
       case 's': ca_racket = 0; break;
       case 'h': usage(); exit(1); break;
       case 'H': host = optarg; break;
@@ -164,12 +168,13 @@ main(int argc, char **argv)
       case 't': timewarp = 1; break;
       case 'l': leap = 1; break;
       case 'x': proxy = optarg; break;
+      case 'w': http = 1; break;
       case '?': break;
       default : fprintf(stderr, "Unknown option!\n"); usage(); exit(1);
     }
   }
 
-  if (verbose) {
+  if (2 == verbose) {
     fprintf(stderr,
       "V: tlsdate version %s\n"
             "V: We were called with the following arguments:\n"
@@ -187,13 +192,14 @@ main(int argc, char **argv)
     port,
     protocol,
     (ca_racket ? "racket" : "unchecked"),
-    (verbose ? "verbose" : "quiet"),
+    (verbose ? (verbose >= 2 ? "debug" : "verbose") : "quiet"),
     ca_cert_container,
     (setclock ? "setclock" : "dont-set-clock"),
-    (showtime ? "showtime" : "no-showtime"),
+    (showtime ? (showtime == 2 ? "showtime=raw" : "showtime") : "no-showtime"),
     (timewarp ? "timewarp" : "no-fun"),
     (leap ? "leapaway" : "holdfast"),
     (proxy ? proxy : "none"),
+    (http ? "http" : "tls"),
     NULL);
   perror("Failed to run tlsdate-helper");
   return 1;
